import { LoggingService } from "../helpers/logging-service";
export class DynamoQueryScanProcessor {
    async __helperDynamoQueryProcessor({ evaluationLimit, params, pageSize, lastKeyHash, orderDesc, hashKeyAndSortKey, dynamoDbClient, }) {
        return await this.__helperDynamoQueryScanProcessor({
            dynamoDbClient,
            operation: "query",
            evaluationLimit,
            params,
            pageSize,
            lastKeyHash,
            orderDesc,
            hashKeyAndSortKey,
        });
    }
    __helperDynamoQueryScanProcessor({ operation, evaluationLimit, params, pageSize, lastKeyHash, orderDesc, hashKeyAndSortKey, dynamoDbClient, }) {
        const xDefaultEvaluationLimit = 10;
        const xMinEvaluationLimit = 5;
        const xMaxEvaluationLimit = 500;
        LoggingService.log({
            processorParamsInit: {
                operation,
                pageSize,
                orderDesc,
                lastKeyHash,
                evaluationLimit,
                hashKeyAndSortKey,
                params,
            },
        });
        return new Promise((resolve, reject) => {
            let returnedItems = [];
            let _evaluationLimit = 0;
            if (pageSize) {
                _evaluationLimit = xDefaultEvaluationLimit;
                if (evaluationLimit) {
                    _evaluationLimit = evaluationLimit;
                }
                if (_evaluationLimit < xMinEvaluationLimit) {
                    _evaluationLimit = xMinEvaluationLimit;
                }
                else if (_evaluationLimit > xMaxEvaluationLimit) {
                    _evaluationLimit = xDefaultEvaluationLimit;
                }
                if (pageSize > _evaluationLimit) {
                    _evaluationLimit = pageSize + 1;
                }
                else if (pageSize === _evaluationLimit) {
                }
            }
            const queryScanUntilDone = (err, data) => {
                if (err) {
                    LoggingService.log(err, err.stack);
                    if (returnedItems?.length) {
                        resolve({ mainResult: returnedItems });
                    }
                    else {
                        reject(err.stack);
                    }
                }
                else {
                    if (data?.Items?.length) {
                        returnedItems = [...returnedItems, ...data.Items];
                    }
                    if (returnedItems.length && hashKeyAndSortKey?.length) {
                        const dataObj = returnedItems.slice(-1)[0];
                        const customLastEvaluationKey = this.__createCustomLastEvaluationKey(dataObj, hashKeyAndSortKey);
                        LoggingService.log({ customLastEvaluationKey });
                    }
                    if (pageSize && returnedItems.length >= pageSize) {
                        const scanResult = {
                            mainResult: returnedItems,
                        };
                        if (data.LastEvaluatedKey &&
                            Object.keys(data.LastEvaluatedKey).length) {
                            const lastKeyHash = this.__encodeLastKey(data.LastEvaluatedKey);
                            scanResult.lastKeyHash = lastKeyHash;
                        }
                        resolve(scanResult);
                    }
                    else if (data.LastEvaluatedKey &&
                        Object.keys(data.LastEvaluatedKey).length) {
                        const _paramsDef = { ...params };
                        _paramsDef.ExclusiveStartKey = data.LastEvaluatedKey;
                        if (_evaluationLimit) {
                            _paramsDef.Limit = _evaluationLimit;
                        }
                        LoggingService.log({
                            operation,
                            dynamoProcessorParams: _paramsDef,
                        });
                        dynamoDbClient()[operation](_paramsDef, queryScanUntilDone);
                    }
                    else {
                        resolve({ mainResult: returnedItems });
                    }
                }
            };
            const _params = { ...params };
            if (_evaluationLimit) {
                _params.Limit = _evaluationLimit;
            }
            if (lastKeyHash) {
                const _lastEvaluatedKey = this.__decodeLastKey(lastKeyHash);
                if (_lastEvaluatedKey) {
                    _params.ExclusiveStartKey = _lastEvaluatedKey;
                }
            }
            if (orderDesc === true && operation === "query") {
                _params["ScanIndexForward"] = false;
            }
            LoggingService.log({ operation, dynamoProcessorParams: _params });
            dynamoDbClient()[operation](_params, queryScanUntilDone);
        });
    }
    __encodeLastKey(lastEvaluatedKey) {
        return Buffer.from(JSON.stringify(lastEvaluatedKey)).toString("base64");
    }
    __createCustomLastEvaluationKey(dataObj, primaryFieldNames) {
        const obj = {};
        primaryFieldNames.forEach((key) => {
            if (typeof dataObj[key] !== "undefined") {
                obj[key] = dataObj[key];
            }
        });
        return Object.keys(obj).length > 0 ? obj : null;
    }
    __decodeLastKey(lastKeyHash) {
        let _lastEvaluatedKey;
        try {
            const _lastKeyHashStr = Buffer.from(lastKeyHash, "base64").toString();
            _lastEvaluatedKey = JSON.parse(_lastKeyHashStr);
        }
        catch (error) {
            _lastEvaluatedKey = undefined;
        }
        return _lastEvaluatedKey;
    }
}
